# -*- coding: utf-8 -*-
import django.dispatch

payment_processed = django.dispatch.Signal()
success_page_visited = django.dispatch.Signal(providing_args=['request', ])
error_page_visited = django.dispatch.Signal(providing_args=['request', ])



# -*- coding: utf-8 -*-

from django.conf.urls.defaults import patterns, url

urlpatterns = patterns('a1lite.views',
    url(r'^process/$', 'process', name='a1lite_process'),
    url(r'^success/$', 'success', name='a1lite_success'),
    url(r'^error/$', 'error', name='a1lite_error'),
 )



# -*- coding: utf-8 -*-
from django.contrib import admin
from django.utils.translation import ugettext_lazy as _

from .models import Payment, PaymentType

class PaymentAdmin(admin.ModelAdmin):
    list_display = ('user', 'is_payed', 'status_verbose', 'created', )
    list_display_links = list_display
    list_filter = ('status', 'payment_type')

admin.site.register(Payment, PaymentAdmin)
admin.site.register(PaymentType)



# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):

        # Changing field 'Payment.tid'
        db.alter_column('a1lite_payment', 'tid', self.gf('django.db.models.fields.CharField')(max_length=32, null=True))

    def backwards(self, orm):

        # Changing field 'Payment.tid'
        db.alter_column('a1lite_payment', 'tid', self.gf('django.db.models.fields.PositiveIntegerField')(null=True))

    models = {
        'a1lite.payment': {
            'Meta': {'object_name': 'Payment'},
            'comment': ('django.db.models.fields.CharField', [], {'max_length': '512', 'null': 'True', 'blank': 'True'}),
            'cost': ('django.db.models.fields.PositiveIntegerField', [], {}),
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'default_email': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '128', 'null': 'True', 'blank': 'True'}),
            'partner_income': ('django.db.models.fields.PositiveIntegerField', [], {'null': 'True', 'blank': 'True'}),
            'payment_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['a1lite.PaymentType']", 'null': 'True', 'blank': 'True'}),
            'phone_number': ('django.db.models.fields.CharField', [], {'max_length': '32', 'null': 'True', 'blank': 'True'}),
            'status': ('django.db.models.fields.PositiveSmallIntegerField', [], {'default': '1'}),
            'system_income': ('django.db.models.fields.PositiveIntegerField', [], {'null': 'True', 'blank': 'True'}),
            'tid': ('django.db.models.fields.CharField', [], {'max_length': '32', 'null': 'True', 'blank': 'True'}),
            'updated': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['auth.User']"})
        },
        'a1lite.paymenttype': {
            'Meta': {'object_name': 'PaymentType'},
            'code': ('django.db.models.fields.CharField', [], {'max_length': '32'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'logo': ('django.db.models.fields.files.ImageField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '512'})
        },
        'auth.group': {
            'Meta': {'object_name': 'Group'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '80'}),
            'permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'})
        },
        'auth.permission': {
            'Meta': {'ordering': "('content_type__app_label', 'content_type__model', 'codename')", 'unique_together': "(('content_type', 'codename'),)", 'object_name': 'Permission'},
            'codename': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contenttypes.ContentType']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        'auth.user': {
            'Meta': {'object_name': 'User'},
            'about': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'age': ('django.db.models.fields.PositiveSmallIntegerField', [], {'default': '0', 'blank': 'True'}),
            'avatar': ('django.db.models.fields.files.ImageField', [], {'max_length': '100', 'blank': 'True'}),
            'birthday': ('django.db.models.fields.DateField', [], {'default': 'None', 'null': 'True', 'blank': 'True'}),
            'date_joined': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'blank': 'True'}),
            'enable_organizer_mode': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'groups': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Group']", 'symmetrical': 'False', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'is_staff': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_superuser': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'last_login': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'organizer': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['events.Organizer']", 'null': 'True', 'blank': 'True'}),
            'password': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'user_permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'}),
            'username': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '30'})
        },
        'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        'events.organizer': {
            'Meta': {'object_name': 'Organizer'},
            'about': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'enable': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'logo': ('django.db.models.fields.files.ImageField', [], {'max_length': '100', 'blank': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '255'})
        }
    }

    complete_apps = ['a1lite']


# -*- coding: utf-8 -*-

from django.core.exceptions import PermissionDenied
from django.http import (
    HttpResponse,
    HttpResponseNotAllowed,
    HttpResponseBadRequest,
)
from django.shortcuts import render_to_response
from django.template import RequestContext
from django.views.decorators.csrf import csrf_exempt

from .models import Payment, PaymentType
from .signals import success_page_visited, error_page_visited

import a1lite_settings

@csrf_exempt
def process(request):
    if request.method != 'POST':
        return HttpResponseNotAllowed('Method not allowed')

    _check_ip(request)

    order_id = request.POST.get('order_id')
    try:
        payment = Payment.objects.get(id=order_id)
    except Payment.DoesNotExist:
        return HttpResponseBadRequest('Payment for order_id=%s not found' % order_id)

    try:
        payment.process(request.POST)
    except Payment.InvalidSignature:
        return HttpResponseBadRequest('Invalid signature')

    return HttpResponse('OK')

def success(request):
    success_page_visited.send(None, request=request)
    return render_to_response('a1lite/success.html',
        context_instance=RequestContext(request))

def error(request):
    error_page_visited.send(None, request=request)
    return render_to_response('a1lite/error.html',
        context_instance=RequestContext(request))


def _check_ip(request):
    a1lite_ips = a1lite_settings.A1LITE_IPS
    if a1lite_ips:
        if (request.META['REMOTE_ADDR'] not in a1lite_ips
            and request.META.get('X_REAL_IP') not in a1lite_ips):
            raise PermissionDenied()



# -*- coding: utf-8 -*-

import sys
from django.conf import settings

def __settings_value(name, default=None):
    setattr(sys.modules[__name__], name, getattr(settings, name, default))

__settings_value('A1LITE_IPS', (
    '78.108.178.206',
    '79.137.235.129',
    '95.163.96.79',
    '212.24.38.100',
))

__settings_value('A1LITE_PAYMENT_TYPE_LOGO_UPLOAD_TO', 'a1lite_logo')
__settings_value('A1LITE_SECRET')
__settings_value('A1LITE_KEY')
__settings_value('A1LITE_SERVICE_ID')



# -*- coding: utf-8 -*-
from setuptools import setup, find_packages

PACKAGE = "a1lite"
NAME = "django_a1lite"
DESCRIPTION = "django application for using http://a1pay.ru/ payment gateway"
AUTHOR = "Dmitry Kuksinsky"
AUTHOR_EMAIL = "dgk@dgk.su"
URL = "https://github.com/dgk/django_a1lite"
VERSION = __import__(PACKAGE).__version__

setup(
    name=NAME,
    version=VERSION,
    description=DESCRIPTION,
    #long_description=read("README.md"),
    author=AUTHOR,
    author_email=AUTHOR_EMAIL,
    license="BSD",
    url=URL,
    packages=[PACKAGE,],
    package_dir={PACKAGE: PACKAGE},
    package_data={PACKAGE: [
        'fixtures/*.json',
        'templates/*/*.html',
        'locale/*/*/django.*',
        'migrations/*.py',
        'templatetags/*.py',
    ]},

    classifiers=[
        "Development Status :: 3 - Alpha",
        "Environment :: Web Environment",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: BSD License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Framework :: Django",
    ],
    zip_safe=False,
)



# -*- coding: utf-8 -*-
from django import template

from .. import a1lite_settings

register = template.Library()

@register.inclusion_tag('a1lite/a1lite_form.html', takes_context=True)
def a1lite_form(context, payment):
    assert (a1lite_settings.A1LITE_KEY
            and a1lite_settings.A1LITE_SERVICE_ID)

    context.update(dict(
        payment=payment,
        a1lite_key=a1lite_settings.A1LITE_KEY,
        a1lite_service_id=a1lite_settings.A1LITE_SERVICE_ID,
    ))
    return context



# -*- coding: utf-8 -*-
import hashlib

from django.db import models
from django.utils.translation import ugettext_lazy as _
from django.contrib.auth.models import User

import a1lite_settings

from .signals import payment_processed

class Payment(models.Model):

    STATUS_PAY_WAITING = 1
    STATUS_PARTIALLY_PAID = 2
    STATUS_CANCELED = 3
    STATUS_EXPIRED = 4
    STATUS_PAYED = 5

    STATUS_CHOICES = (
        (STATUS_PAY_WAITING, _('Pay waiting')),
        (STATUS_CANCELED, _('Canceled')),
        (STATUS_EXPIRED, _('Expired')),
        (STATUS_PARTIALLY_PAID, _('Partially paid')),
        (STATUS_PAYED, _('Payed')),
    )

    user = models.ForeignKey(User, verbose_name=_('User'))
    status = models.PositiveSmallIntegerField(verbose_name=_('Status'),
        choices=STATUS_CHOICES, default=STATUS_PAY_WAITING)

    cost = models.DecimalField(max_digits=12, decimal_places=2,
        verbose_name=_('Cost'))

    default_email = models.EmailField(verbose_name=_('Email'),
        null=True, blank=True)
    phone_number = models.CharField(verbose_name=_('Phone number'),
        max_length=32, null=True, blank=True)

    name = models.CharField(verbose_name=_('Name'),
        help_text=_('Name of product or service'),
        max_length=128, null=True, blank=True)
    comment = models.CharField(verbose_name=_('Comment'),
        help_text=_('Payment comment'),
        max_length=512, null=True, blank=True)

    tid = models.CharField(verbose_name=_('Transaction ID'),
        max_length=32,
        null=True, blank=True)
    payment_type = models.ForeignKey('PaymentType', verbose_name=_('Payment type'),
        null=True, blank=True)

    partner_income = models.DecimalField(max_digits=12, decimal_places=2,
        verbose_name=_('Partner income'),
        null=True, blank=True)
    system_income = models.DecimalField(max_digits=12, decimal_places=2,
        verbose_name=_('System income'),
        null=True, blank=True)

    created = models.DateTimeField(verbose_name=_('Created'), auto_now_add=True)
    updated = models.DateTimeField(verbose_name=_('Updated'), auto_now=True)

    class Meta():
        verbose_name=_('Payment')
        verbose_name_plural=_('Payments')

    class InvalidSignature(Exception):
        pass

    def __unicode__(self):
        return '%d (%d) / %s' % (self.id, self.cost, unicode(self.status_verbose))

    @property
    def status_verbose(self):
        return [ x[1] for x in self.STATUS_CHOICES
                               if x[0] == self.status ][0]

    def is_payed(self):
        return self.status == self.STATUS_PAYED

    def process(self, data_dict):
        keys = [
                'tid',
                'name',
                'comment',
                'partner_id',
                'service_id',
                'order_id',
                'type',
                'partner_income',
                'system_income'
            ]

        if 'test' in data_dict:
            keys.append('test')

        params = dict([(x, data_dict.get(x, '')) for x in keys ])

        check = hashlib.md5((''.join([params[x] for x in keys ])
                             + a1lite_settings.A1LITE_SECRET).encode('utf8')).hexdigest()

        if check != data_dict.get('check'):
            raise Payment.InvalidSignature()

        if float(params['system_income']) < self.cost:
            self.status = Payment.STATUS_PARTIALLY_PAID
        else:
            self.status = Payment.STATUS_PAYED

        for k in keys:
            if k in ('type', 'order_id', 'test', ):
                continue
            setattr(self, k, params[k])

        self.payment_type = PaymentType.objects.get(code=params['type'])
        self.save()

        payment_processed.send(self)

class PaymentType(models.Model):
    code = models.CharField(max_length=32)
    title = models.CharField(max_length=512)
    logo = models.ImageField(null=True, blank=True,
        upload_to=a1lite_settings.A1LITE_PAYMENT_TYPE_LOGO_UPLOAD_TO)

    class Meta():
        verbose_name=_('Payment type')
        verbose_name_plural=_('Payment types')

    def __unicode__(self):
        return self.title
